@@ -54,6 +54,15 @@
 #' places to display for the *p*-value.
 #' Default is 3.
 #'
+#' @param se Logical. If `TRUE` and
+#' confidence interval is available, the
+#' standard error of the estimate is
+#' also printed. This is simply the
+#' standard deviation of the bootstrap
+#' estimates or Monte Carlo simulated
+#' values, depending on the method used
+#' to form the confidence interval.
+#'
 #' @param ... Other arguments. Not used.
 #'
 #'
@@ -115,6 +124,7 @@ print.indirect <- function(x,
                            digits = 3,
                            pvalue = FALSE,
                            pvalue_digits = 3,
+                           se = FALSE,
                            ...) {
     xold <- x
     my_call <- x$call
@@ -124,16 +134,23 @@ print.indirect <- function(x,
     standardized <- (standardized_x && standardized_y)
     has_ci <- FALSE
     ci_type <- NULL
+    if (is.numeric(x$group_number)) {
+        has_group <- TRUE
+      } else {
+        has_group <- FALSE
+      }
     if (isTRUE(!is.null(x$boot_ci))) {
         has_ci <- TRUE
         ci_type <- "boot"
         ci_name <- "boot_ci"
+        se_name <- "boot_se"
         R <- length(x$boot_indirect)
       }
     if (isTRUE(!is.null(x$mc_ci))) {
         has_ci <- TRUE
         ci_type <- "mc"
         ci_name <- "mc_ci"
+        se_name <- "mc_se"
         R <- length(x$mc_indirect)
       }
     has_w <- isTRUE(!is.null(wvalues))
@@ -182,6 +199,11 @@ print.indirect <- function(x,
       } else {
         path <- paste(x0, "->", y0)
       }
+    if (has_group) {
+        path <- paste0(x$group_label, "[",
+                       x$group_number, "]: ",
+                       path)
+      }
     std_str <- ""
     if (standardized) {
         std_str <- paste0("(Both ", sQuote(x0),
@@ -204,7 +226,8 @@ print.indirect <- function(x,
         cat("\n== Conditional", cond_str2, "Effect",
             std_str, " ==")
       } else {
-        cat("\n==", cond_str2, "Effect ==")
+        cat("\n==", cond_str2, "Effect",
+            std_str, "==")
       }
     cat("\n")
     ptable <- data.frame(Factor = "Path:", Value = path)
@@ -238,6 +261,18 @@ print.indirect <- function(x,
           } else {
             b_row2 <- NULL
           }
+        if (se) {
+            tmpp <- ifelse(!is.null(x[[se_name]]) && is.numeric(x[[se_name]]),
+                           formatC(x[[se_name]], digits = digits, format = "f"),
+                           "Not available"
+                           )
+            tmp1 <- switch(ci_type,
+                           boot = "Bootstrap SE:",
+                           mc = "Monte Carlo SE:")
+            b_row3 <- c(tmp1, tmpp)
+          } else {
+            b_row3 <- NULL
+          }
       }
     if (has_w) {
         if (is.null(x$op)) {
@@ -253,20 +288,26 @@ print.indirect <- function(x,
         tmp <- paste(paste(wnames, "=", formatC(w0,
                                                 digits = digits,
                                                 format = "f")), collapse = ", ")
-        if (has_ci) {ptable <- rbind(ptable, b_row, b_row2)}
+        if (has_ci) {ptable <- rbind(ptable, b_row, b_row2, b_row3)}
         ptable <- rbind(ptable,
                         c("When:", tmp))
       } else {
         if (is.null(x$op)) {
             ptable <- rbind(ptable,
-                            c(ifelse(has_m, "Indirect Effect", "Effect"),
+                            c(ifelse(has_m, "Indirect Effect:", "Effect:"),
                               formatC(x$indirect, digits = digits, format = "f")))
           } else {
             ptable <- rbind(ptable,
                             c("Function of Effects:",
                               formatC(x$indirect, digits = digits, format = "f")))
           }
-        if (has_ci) {ptable <- rbind(ptable, b_row)}
+        if (has_ci) {ptable <- rbind(ptable, b_row, b_row2, b_row3)}
+      }
+    if (has_group) {
+        # ptable <- rbind(ptable,
+        #                 c("Group Label:", x$group_label))
+        # ptable <- rbind(ptable,
+        #                 c("Group Number:", x$group_number))
       }
     ptable <- data.frame(lapply(ptable, format))
     colnames(ptable) <- c("", "")
@@ -305,7 +346,26 @@ print.indirect <- function(x,
                           R,
                           " replications."))
         cat(strwrap(tmp1), sep = "\n")
+        if (se) {
+            tmp1 <- switch(ci_type,
+                      boot = paste("Standard error (SE) based on nonparametric bootstrapping",
+                              "with ",
+                              R,
+                              " bootstrap samples."),
+                      mc = paste("Standard error (SE) based on Monte Carlo simulation",
+                              "with ",
+                              R,
+                              " replications."))
+            cat(strwrap(tmp1), sep = "\n")
+          }
       }
+    print_note <- FALSE
+    if (standardized_x ||
+        standardized_y ||
+        has_group) {
+        print_note <- TRUE
+      }
+    note_str <- character(0)
     if (has_m & !is.list(mpathnames)) {
         if (has_w) {
           out <- data.frame(mpathnames, m0c, m0)
@@ -320,8 +380,27 @@ print.indirect <- function(x,
         cat("\n")
         print(out, digits = digits, row.names = FALSE)
         if (standardized_x || standardized_y) {
-          cat("\nNOTE: The effects of the component paths are from the model, not standardized.")
-        }
+            note_str <- c(note_str,
+                  strwrap("- The effects of the component paths are from the model, not standardized.",
+                          exdent = 2))
+            if (has_group) {
+                note_str <- c(note_str,
+                  strwrap("- SD(s) in the selected group is/are used in standardiziation.",
+                          exdent = 2))
+              }
+          }
+      }
+    if (has_group) {
+      note_str <- c(note_str,
+          strwrap("- The group label is printed before each path.",
+                  exdent = 2))
+      note_str <- c(note_str,
+          strwrap("- The group number in square brackets is the number used internally in lavaan.",
+                  exdent = 2))
+    }
+    if (length(note_str) > 0) {
+        cat("\nNOTE:\n")
+        cat(note_str, sep = "\n")
       }
     cat("\n")
     invisible(x)
